# Use K-cross function to explore interaction among placenames and
# to perform 95% Monte Carlo test as described in Section 2.3.1 of Zanella (2014)
# it needs a numerical vector called "included" that indicates which placenames to consider

directory<-''#address of the folder were the R files are stored (if it is not the home directory)
source(paste(directory,'data_acquisition_and_transformation.R',sep=''))
source(paste(directory,'functions.R',sep=''))

#### CREATE POINTS LIST AND DATA-FRAME #########
###
names_included<-names[included]
k<-length(names_included)
points<-list()
points[[k]]<-NA
n_points<-rep(0,k+1)
all_points<-NULL
marks_list<-as.character(1:k)
marks<-c()
for (i in 1:k){
  points[[i]]<-t(points_list_NC[[included[i]]])
  n_points[(i+1):(k+1)]<-n_points[(i+1):(k+1)]+length(points[[i]])/2
  all_points<-cbind(all_points,points[[i]])
  marks<-c(marks,rep(marks_list[i],length(points[[i]])/2))
}
marked_NC.df<-data.frame(x=all_points[1,],y=all_points[2,],marks=marks)

#### CREATE POINT PROCESS OBJECT (SELECT REGION WITH DENSITY ABOVE A TRESHOLD AND INSIDE UK) ##########
###
n_grid<-200;border<-10e03
x_min<-min(marked_NC.df$x)-border
x_max<-max(marked_NC.df$x)+border
y_min<-min(marked_NC.df$y)-border
y_max<-max(marked_NC.df$y)+border
marked_NC.ppp<-as.ppp(marked_NC.df,W=owin(xrange=c(x_min,x_max),yrange=c(y_min,y_max)))
centre_density<-density(x=marked_NC.ppp,sigma=bw.relrisk(marked_NC.ppp,method='likelihood'),dimyx=c(n_grid,n_grid),edge=FALSE)
centre_density<-cut_density(centre_density=centre_density,n_grid=n_grid)
window<-owin(xrange=c(x_min,x_max),yrange=c(y_min,y_max),mask=centre_density$v>1e-20)
marked_NC.ppp<-as.ppp(marked_NC.df,W=window)

#### PARAMETERS FOR THE MONTE-CARLO TEST #######
###
r_max<-15e03
n_sim<-99
nrank<-5
n_point_r<-20
r<-c((c(0:n_point_r)/n_point_r)*r_max,156400)

#### DEFINES K_CROSS-FUNCTION all-to-all (SEE SECTION 2 OF (Zanella 2014)) ########
#### AND CORRESPONDING L-CROSS FUNCTION #########
###
K.cross<-function(X,r=NULL){
  marks_list<-levels(X$marks)
  k<-length(marks_list)
  list.ppp<-split(X)
  intensity.list<-lapply(list.ppp,density,sigma=bw.relrisk(X))
  num_points<-rep(NA,k)
  for (i in 1:k){
    num_points[i]<-sum(X$marks==marks_list[i])
  }
  Kcross.inhom.est.list<-as.list(rep(NA,k))
  ind<-0
  norm_const<-0
  for (i in 1:k){
    for (j in (1:k)){
      if (i<j){
        ind<-ind+1
        norm_const<-norm_const+num_points[i]*num_points[j]
        Kcross.inhom.est.list[[ind]]<-Kcross.inhom(X=X,r=r,i=marks_list[i],j=marks_list[j],
                                                   lambdaI=intensity.list[[i]],lambdaJ=intensity.list[[j]],correction="border")
      }
    }
  }
  K.cross<-Kcross.inhom.est.list[[1]]
  K.cross$border<-K.cross$border-K.cross$border
  ind<-0
  for (i in 1:k){
    for (j in (1:k)){
      if (i<j){
        ind<-ind+1
        K.cross$border<-K.cross$border+
          (num_points[i]*num_points[j]/norm_const) * Kcross.inhom.est.list[[ind]]$border
      }
    }
  }
  return(K.cross)
}
L.cross<-function(X,r=NULL){
  L.cross<-K.cross(X=X,r=r)
  L.cross$border<-sqrt(L.cross$border/pi)-L.cross$r
  L.cross$theo<-NULL
  return(L.cross)
}

#### ENVELOPES AND DEVIANCE TEST WITH NULL HYPOTHESIS #####
####  BEING A MULTI-TYPE STRAUSS POINT PROCESS        #####
####     AS IN SECTION 2 OF Zanella (2014)            #####
###
bandwidth_used<-bw.relrisk(marked_NC.ppp)
rel_risk_list<-relrisk(marked_NC.ppp,casecontrol=FALSE)
glob.intensity<-density(x=marked_NC.ppp,sigma=bandwidth_used,diggle=TRUE)
intensities<-rel_risk_list
for (i in 1:k){
  intensities[[i]]$v<-rel_risk_list[[i]]$v*glob.intensity$v
}
nr  <- 1e6
nv  <- 500000
generate_sample_under_null<-function(){
  k<-length(intensities)
  straussm_par<-list(beta=rep(1,k),gamma=1-diag(rep(1-gamma,k)),radii=diag(rep(R,k)))
  null_model <- list(cif="straussm",par=straussm_par,w=window,trend=intensities,types=marks_list)
  return(rmh(model=null_model,start=list(n.start=1),control=list(nrep=nr,nverb=nv)))
}
env<-envelope(Y=marked_NC.ppp,nsim=n_sim,nrank=nrank,
              fun=L.cross,simulate=expression(generate_sample_under_null()))

#### CENTER THE ENVELOPES AND CONVERT FROM METERS TO KM ########
###
env.centred<-env
env.centred$obs<-(env$obs-env$mmean)/1e03
env.centred$lo<-(env$lo-env$mmean)/1e03
env.centred$hi<-(env$hi-env$mmean)/1e03
env.centred$mmean<-env.centred$mmean-env.centred$mmean
env.centred$r<-env.centred$r/1e03
r_max<-r_max/1e03