# This file performs the Convergence Diagnotistic of Section 4.1.2 of the paper (Zanella 2014)
# It outputs the two figures of Section 4.1.2 and the results of the table of of Section 4.1.2 

directory<-''#address of the folder were the R files are stored (if it is not the home directory)
source(paste(directory,'functions.R',sep=''))

### TIME PARAMETERS TO REGULATE #####
###
T_links_count_ref<-10^6;burn_time_links_count_ref<-10^3
T_ref_match<-10^3
T_start_match<-10^3
burn_time<-10^3;T<-2*10^5 #length of the  MCMC chains to diagnose

### EVALUATE POINT CONFIGURATIONS AND RELATIVE WEIGHTS MATRIX #############
####
W<-10*10
sigma<-0.3
lambda<-50
p_c<-c(0.5,0.5)
xy<-poisson_model_synt_sample(W=W,sigma=sigma,lambda=lambda,p_c=p_c)#Plot_sample(xy)
weights<-evaluate_weights_poisson_2d(xy=xy,sigma=sigma,lambda=lambda,W=W,p_c=p_c)

### EVALUATE START MATCHINGS, REFERENCE LINKS COUNT AND REFERENCE MATCHINGS ##########
###
print("Evaluating reference and starting configurations")
links_count_ref<-twod_MH_P4(weights=weights,T=T_links_count_ref,burn_time=burn_time_links_count_ref,eval_links_count=TRUE)$links_count
ref_match_list<-list()
for (i in 1:10){
  ref_match_list[[i]]<-twod_MH_P4(weights=weights,T=T_ref_match,burn_time=0)$matching
}
start_match_list<-list()
for (i in 1:5){
  start_match_list[[i]]<-twod_MH_P4(weights=weights,T=T_start_match,burn_time=0)$matching
}

### RUNNING CHAINS ##########
###
print("Running MCMC chains")
trunc_tresh<-0.001;print_time=FALSE
output_P1_list<-list();output_P2_list<-list();output_P3_list<-list();output_P4_list<-list()
for (iter in 1:5){
  output_P1_list[[iter]]<-twod_MH_P1(weights=weights,start_match=start_match_list[[iter]],T=T,burn_time=burn_time,trunc_tresh=trunc_tresh,print_time=print_time,ref_match_list=ref_match_list,eval_links_count=TRUE,links_count_ref=links_count_ref)
  output_P2_list[[iter]]<-twod_MH_P2(weights=weights,start_match=start_match_list[[iter]],T=T,burn_time=burn_time,print_time=print_time,ref_match_list=ref_match_list,eval_links_count=TRUE,links_count_ref=links_count_ref)
  output_P3_list[[iter]]<-twod_MH_P3(weights=weights,start_match=start_match_list[[iter]],T=T,burn_time=burn_time,print_time=print_time,ref_match_list=ref_match_list,eval_links_count=TRUE,links_count_ref=links_count_ref)
  output_P4_list[[iter]]<-twod_MH_P4(weights=weights,start_match=start_match_list[[iter]],T=T,burn_time=burn_time,print_time=print_time,ref_match_list=ref_match_list,eval_links_count=TRUE,links_count_ref=links_count_ref)
}

### EVALUATING TIME TO MAKE 10^4 STEPS ##########
###
#NB: this time is for plain mcmc without the convergence diagnostic (i.e. without evaluating links count and differences from a reference matching list)
time_P1<-system.time(twod_MH_P1(weights=weights,T=10^4,burn_time=0,trunc_tresh=trunc_tresh))
time_P2<-system.time(twod_MH_P2(weights=weights,T=10^4,burn_time=0))
time_P3<-system.time(twod_MH_P3(weights=weights,T=10^4,burn_time=0))
time_P4<-system.time(twod_MH_P4(weights=weights,T=10^4,burn_time=0))

### BUILD MCMC OBJECTS FOR PACKAGE CODA ########
###
print("Performing convergence diagnostic")
mcmc_list_P1<-list();mcmc_list_P2<-list();mcmc_list_P3<-list();mcmc_list_P4<-list()
for (iter in 1:5){
  mcmc_list_P1[[iter]]<-as.mcmc(t(output_P1_list[[iter]]$diff_from_ref))
  mcmc_list_P2[[iter]]<-as.mcmc(t(output_P2_list[[iter]]$diff_from_ref))
  mcmc_list_P3[[iter]]<-as.mcmc(t(output_P3_list[[iter]]$diff_from_ref))
  mcmc_list_P4[[iter]]<-as.mcmc(t(output_P4_list[[iter]]$diff_from_ref))
}
mcmc_list_P1<-as.mcmc.list(mcmc_list_P1);mcmc_list_P2<-as.mcmc.list(mcmc_list_P2);mcmc_list_P3<-as.mcmc.list(mcmc_list_P3);mcmc_list_P4<-as.mcmc.list(mcmc_list_P4)

### ESTIMATE AUTOCORRELATION, EFFECTIVE SAMPLE SIZE AND INTEGRATED AUTOCORRELATION TIME ####
###
ESS_P1<-mean(effectiveSize(window(x=mcmc_list_P1,start=burn_time,end=burn_time+T,thin=1)))
ESS_P2<-mean(effectiveSize(window(x=mcmc_list_P2,start=burn_time,end=burn_time+T,thin=1)))
ESS_P3<-mean(effectiveSize(window(x=mcmc_list_P3,start=burn_time,end=burn_time+T,thin=1)))
ESS_P4<-mean(effectiveSize(window(x=mcmc_list_P4,start=burn_time,end=burn_time+T,thin=1)))
lag.max=100
ACF_P1<-rep(0,lag.max+1);ACF_P2<-rep(0,lag.max+1);ACF_P3<-rep(0,lag.max+1);ACF_P4<-rep(0,lag.max+1)
for (iter in 1:5){
  for (iter2 in 1:10){
    ACF_P1<-ACF_P1+c(acf(output_P1_list[[iter]]$diff_from_ref[iter2,burn_time:(burn_time+T)],lag.max=lag.max,plot=FALSE)$acf)
    ACF_P2<-ACF_P2+c(acf(output_P2_list[[iter]]$diff_from_ref[iter2,burn_time:(burn_time+T)],lag.max=lag.max,plot=FALSE)$acf)
    ACF_P3<-ACF_P3+c(acf(output_P3_list[[iter]]$diff_from_ref[iter2,burn_time:(burn_time+T)],lag.max=lag.max,plot=FALSE)$acf)
    ACF_P4<-ACF_P4+c(acf(output_P4_list[[iter]]$diff_from_ref[iter2,burn_time:(burn_time+T)],lag.max=lag.max,plot=FALSE)$acf)
  }
}
ACF_P1<-ACF_P1/50;ACF_P2<-ACF_P2/50;ACF_P3<-ACF_P3/50;ACF_P4<-ACF_P4/50
acf.treshold<-0.01
ACF_P1<-c(ACF_P1,0);ACF_P2<-c(ACF_P2,0);ACF_P3<-c(ACF_P3,0);ACF_P4<-c(ACF_P4,0)
IAT_P1<-1+2*sum(ACF_P1[1:(which(ACF_P1<acf.treshold)[1])])
IAT_P2<-1+2*sum(ACF_P2[1:(which(ACF_P2<acf.treshold)[1])])
IAT_P3<-1+2*sum(ACF_P3[1:(which(ACF_P3<acf.treshold)[1])])
IAT_P4<-1+2*sum(ACF_P4[1:(which(ACF_P4<acf.treshold)[1])])

### EVALUATE D VECTOR #####
###
D_vec_P1<-colMeans(rbind(output_P1_list[[1]]$D_vec,output_P1_list[[2]]$D_vec,output_P1_list[[3]]$D_vec,output_P1_list[[4]]$D_vec,output_P1_list[[5]]$D_vec))
D_vec_P2<-colMeans(rbind(output_P2_list[[1]]$D_vec,output_P2_list[[2]]$D_vec,output_P2_list[[3]]$D_vec,output_P2_list[[4]]$D_vec,output_P2_list[[5]]$D_vec))
D_vec_P3<-colMeans(rbind(output_P3_list[[1]]$D_vec,output_P3_list[[2]]$D_vec,output_P3_list[[3]]$D_vec,output_P3_list[[4]]$D_vec,output_P3_list[[5]]$D_vec))
D_vec_P4<-colMeans(rbind(output_P4_list[[1]]$D_vec,output_P4_list[[2]]$D_vec,output_P4_list[[3]]$D_vec,output_P4_list[[4]]$D_vec,output_P4_list[[5]]$D_vec))

### EVALUATE MULTIVARIATE GELMAN AND RUBIN DIAGNOSTIC ####
###
plot_step<-T/100
T_vec<-c(1:(T/plot_step))*plot_step
GR_vec_P1<-c();GR_vec_P2<-c();GR_vec_P3<-c();GR_vec_P4<-c()
for (T_temp in T_vec){
  GR_vec_P1<-c(GR_vec_P1,gelman.diag(x=window(x=mcmc_list_P1,start=burn_time,end=burn_time+T_temp,thin=1),autoburnin=FALSE)$mpsrf)
  GR_vec_P2<-c(GR_vec_P2,gelman.diag(x=window(x=mcmc_list_P2,start=burn_time,end=burn_time+T_temp,thin=1),autoburnin=FALSE)$mpsrf)
  GR_vec_P3<-c(GR_vec_P3,gelman.diag(x=window(x=mcmc_list_P3,start=burn_time,end=burn_time+T_temp,thin=1),autoburnin=FALSE)$mpsrf)
  GR_vec_P4<-c(GR_vec_P4,gelman.diag(x=window(x=mcmc_list_P4,start=burn_time,end=burn_time+T_temp,thin=1),autoburnin=FALSE)$mpsrf)
}

#######################################
######## PRINTING RESULTS #############
#######################################

print(paste("Time to make 10^4 steps for for P1:",round(time_P1[[1]],digits=2)))
print(paste("Time to make 10^4 steps for for P2:",round(time_P2[[1]],digits=2)))
print(paste("Time to make 10^4 steps for for P3:",round(time_P3[[1]],digits=2)))
print(paste("Time to make 10^4 steps for for P4:",round(time_P4[[1]],digits=2)))
print(paste("Average ESS for P1:",ESS_P1 ))
print(paste("Average ESS for P2:",ESS_P2 ))
print(paste("Average ESS for P3:",ESS_P3 ))
print(paste("Average ESS for P4:",ESS_P4 ))
print(paste("Average IAT for P1:",IAT_P1))
print(paste("Average IAT for P2:",IAT_P2))
print(paste("Average IAT for P3:",IAT_P3))
print(paste("Average IAT for P4:",IAT_P4))
D_treshold<-0.05
print(paste("Time to D<",D_treshold,"for P1 is:",which(D_vec_P1<D_treshold)[1]))
print(paste("Time to D<",D_treshold,"for P2 is:",which(D_vec_P2<D_treshold)[1]))
print(paste("Time to D<",D_treshold,"for P3 is:",which(D_vec_P3<D_treshold)[1]))
print(paste("Time to D<",D_treshold,"for P4 is:",which(D_vec_P4<D_treshold)[1]))
GR_treshold<-1.005
print(paste("Time to GR<",GR_treshold,"for P1 is:",T_vec[which(GR_vec_P1<GR_treshold)[1]]))
print(paste("Time to GR<",GR_treshold,"for P2 is:",T_vec[which(GR_vec_P2<GR_treshold)[1]]))
print(paste("Time to GR<",GR_treshold,"for P3 is:",T_vec[which(GR_vec_P3<GR_treshold)[1]]))
print(paste("Time to GR<",GR_treshold,"for P4 is:",T_vec[which(GR_vec_P4<GR_treshold)[1]]))

#######################################
######## PLOTTING RESULTS #############
#######################################

col_P1<-'black';col_P2<-'blue';col_P3<-'red';col_P4<-'green'
par(mfrow=c(2,2))
par(mgp=c(1.6,0.3,0))
par(mar=c(0,3,2.5,0.05))
T_max<-min(T,5*10^4)
par(mar=c(0,3,2.5,0.05))
ymin<-min(output_P1_list[[1]]$diff_from_ref[1,1:T_max],output_P2_list[[1]]$diff_from_ref[1,1:T_max],output_P3_list[[1]]$diff_from_ref[1,1:T_max],output_P4_list[[1]]$diff_from_ref[1,1:T_max])
ymax<-max(output_P1_list[[1]]$diff_from_ref[1,1:T_max],output_P2_list[[1]]$diff_from_ref[1,1:T_max],output_P3_list[[1]]$diff_from_ref[1,1:T_max],output_P4_list[[1]]$diff_from_ref[1,1:T_max])
plot(ylim=c(ymin,ymax),output_P1_list[[1]]$diff_from_ref[1,1:T_max],xaxt='n', ann=T,type='l',col=col_P1,ylab="Summary statistic",xlab="Markov chain time")
title(main="Traceplots of a summary statistic for different proposals",outer=TRUE,line=-1.5,cex.main=1.5)
legend(legend="P1",col=col_P1,lty=1,x='topright')
par(mar=c(0,3,2.5,0.05))
plot(ylim=c(ymin,ymax),output_P2_list[[1]]$diff_from_ref[1,1:T_max],xaxt='n', ann=T,type='l',col=col_P2,ylab="Summary statistic")
par(mar=c(2.5,3,0,0.05))
legend(legend="P2",col=col_P2,lty=1,x='topright')
plot(ylim=c(ymin,ymax),output_P3_list[[1]]$diff_from_ref[1,1:T_max],type='l',col=col_P3,ylab="Summary statistic",xlab="Markov chain time")
par(mar=c(2.5,3,0,0.05))
legend(legend="P3",col=col_P3,lty=1,x='topright')
plot(ylim=c(ymin,ymax),output_P4_list[[1]]$diff_from_ref[1,1:T_max],type='l',col=col_P4,ylab="Summary statistic",xlab="Markov chain time")
legend(legend="P4",col=col_P4,lty=1,x='topright')


pch_P1<-5;pch_P2<-3;pch_P3<-4;pch_P4<-2
par(mar=c(4.5,2,1.2,1.5),mfrow=c(2,2))
Plot_matching_gray(xy=xy,links_count=links_count_ref,lwd=2)
title("Posterior matching distribution")
title(sub="(a)")

par(mar=c(4.5,4,1.2,0.8))
num_points<-25
thinned_indices<-(1:num_points)*floor(length(ACF_P1)/num_points)
plot(ACF_P1[1:lag.max],type='l',col=col_P1,ylim=c(-0.1,1),#,ylim=c(-0.1,1)
     main="Estimated autocorrelation function",xlab='lag',ylab="Estimated autocorrelation")
lines(ACF_P2[1:lag.max],col=col_P2)
lines(ACF_P3[1:lag.max],col=col_P3)
lines(ACF_P4[1:lag.max],col=col_P4)
points(thinned_indices,ACF_P1[thinned_indices],col=col_P1,pch=pch_P1)
points(thinned_indices,ACF_P2[thinned_indices],col=col_P2,pch=pch_P2)
points(thinned_indices,ACF_P3[thinned_indices],col=col_P3,pch=pch_P3)
points(thinned_indices,ACF_P4[thinned_indices],col=col_P4,pch=pch_P4)
lines(c(0,lag.max),c(0,0),lty=2)
legend(x='topright',legend=c('P1','P2','P3','P4'),
       lty=c(1,1,1,1),col=c(col_P1,col_P2,col_P3,col_P4),
       pch=c(pch_P1,pch_P2,pch_P3,pch_P4))
title(sub="(b)")

par(mar=c(3.6,2.7,2,1))
num_points<-25
y_max<-0.2
thinned_indices<-(1:num_points)*floor(length(D_vec_P1)/num_points)
plot(c(0,T+burn_time),c(0,0),type='l',lty=2,ylim=c(0,y_max),
     main="Diagnostic statistic D",xlab="MCMC time",ylab="Value of D")
lines(which(D_vec_P1<y_max),D_vec_P1[D_vec_P1<y_max],col=col_P1)
points(thinned_indices[D_vec_P1[thinned_indices]<y_max],
       D_vec_P1[thinned_indices][D_vec_P1[thinned_indices]<y_max],col=col_P1,pch=pch_P1)
lines(which(D_vec_P2<y_max),D_vec_P2[D_vec_P2<y_max],col=col_P2)
points(thinned_indices[D_vec_P2[thinned_indices]<y_max],
       D_vec_P2[thinned_indices][D_vec_P2[thinned_indices]<y_max],col=col_P2,pch=pch_P2)
lines(which(D_vec_P3<y_max),D_vec_P3[D_vec_P3<y_max],col=col_P3)
points(thinned_indices[D_vec_P3[thinned_indices]<y_max],
       D_vec_P3[thinned_indices][D_vec_P3[thinned_indices]<y_max],col=col_P3,pch=pch_P3)
lines(which(D_vec_P4<y_max),D_vec_P4[D_vec_P4<y_max],col=col_P4)
points(thinned_indices[D_vec_P4[thinned_indices]<y_max],
       D_vec_P4[thinned_indices][D_vec_P4[thinned_indices]<y_max],col=col_P4,pch=pch_P4)
legend(x='topright',legend=c('P1','P2','P3','P4'),
       lty=c(1,1,1,1),col=c(col_P1,col_P2,col_P3,col_P4),
       pch=c(pch_P1,pch_P2,pch_P3,pch_P4))
title(sub="(c)")

par(mar=c(3.6,4,2,0.8))
num_points<-25
thinned_indices<-(1:num_points)*floor(length(GR_vec_P1)/num_points)
y_max<-1.02
plot(T_vec,rep(1,length(T_vec)),type='l',lty=2,ylim=c(1,y_max),
     main="G&R statistic",xlab="MCMC time",ylab="Gelman and Rubin statistic")
y_max<-1.021
lines(T_vec[GR_vec_P1<y_max],GR_vec_P1[GR_vec_P1<y_max],col=col_P1)
points(T_vec[thinned_indices][GR_vec_P1[thinned_indices]<y_max],
       GR_vec_P1[thinned_indices][GR_vec_P1[thinned_indices]<y_max],col=col_P1,pch=pch_P1)
lines(T_vec[GR_vec_P2<y_max],GR_vec_P2[GR_vec_P2<y_max],col=col_P2)
points(T_vec[thinned_indices][GR_vec_P2[thinned_indices]<y_max],
       GR_vec_P2[thinned_indices][GR_vec_P2[thinned_indices]<y_max],col=col_P2,pch=pch_P2)
lines(T_vec[GR_vec_P3<y_max],GR_vec_P3[GR_vec_P3<y_max],col=col_P3)
points(T_vec[thinned_indices][GR_vec_P3[thinned_indices]<y_max],
       GR_vec_P3[thinned_indices][GR_vec_P3[thinned_indices]<y_max],col=col_P3,pch=pch_P3)
lines(T_vec[GR_vec_P4<y_max],GR_vec_P4[GR_vec_P4<y_max],col=col_P4)
points(T_vec[thinned_indices][GR_vec_P4[thinned_indices]<y_max],
       GR_vec_P4[thinned_indices][GR_vec_P4[thinned_indices]<y_max],col=col_P4,pch=pch_P4)
legend(x='topright',legend=c('P1','P2','P3','P4'),
       lty=c(1,1,1,1),col=c(col_P1,col_P2,col_P3,col_P4),
       pch=c(pch_P1,pch_P2,pch_P3,pch_P4))
title(sub="(d)")
par(mfrow=c(1,1))