# This file runs the MCMC to explore the posterior resulting from fitting the
# Poisson model of Section 2 of (Zanella 2014) to the Anglo-Saxon settlements data 
# (the selected version with 13 placenames). See Section 5.3 of (Zanella 2014) for results.

directory<-''#address of the folder were the R files are stored (if it is not the home directory)
source(paste(directory,'data_acquisition_and_transformation.R',sep=''))
source(paste(directory,'functions.R',sep=''))

#### SELECT PLACENAMES AND CREATE "points" LIST #########
###
included<-c(1,4,6,9,10,11,12,14,15,16,17,19,20)
names_included<-names[included]
k<-length(names_included)
points<-list()
points[[k]]<-NA
n_points<-rep(0,k+1)
for (i in 1:k){
  points[[i]]<-t(points_list_NC[[included[i]]])
  n_points[(i+1):(k+1)]<-n_points[(i+1):(k+1)]+length(points[[i]])/2
}

#### CHOOSE BETWEEN UNIFORM AND NON-UNIFORM MARKS ASSUPTION #########
###
marks_probabilities<-rep(1/k,k)  # uniform marks assumption
# un-comment the following line to use the non-uniform marks assuption (with fixed marks probabilities)
# marks_probabilities<-(n_points[2:(k+1)]-n_points[1:k])/sum(n_points[2:(k+1)]-n_points[1:k])  
marks_norm_consts<-rep(NA,k)
for (s in 1:k){
  marks_norm_consts[s]<-permanentRyser(matrix(c(rep(marks_probabilities,s),rep(1,(k-s)*k)),k,k))/(factorial(s)*factorial(k-s))
}

######### RUN MCMC AND EXPLORE POSTERIOR  ###############
###
start_p_c=c(0.6,0.4-sum(rep(0.01,k-2)),rep(0.01,k-2));start_sigma=4e03;start_lambda=n_points[k+1]/1.4
output<-kd_MH_poisson_model(points=points,n_points=n_points,T=10^6,burn_time=10^4,border=10*10^3,
      n_grid=400,thinning=10,num_steps_in_2d=200,sampler_2d=sampler_2d_P4,
      start_p_c=start_p_c,start_sigma=start_sigma,start_lambda=start_lambda,
      edge_correction=TRUE,eval_links_count=TRUE,eval_association_count=TRUE)
output$sigma_vec<-output$sigma_vec*sqrt(pi)/10^3

##### A FEW TRACEPLOTS ####################
###
plot(output$sigma_vec,type="l",ylim=c(0,15),
     xlab='MCMC time(thinned)',ylab="intra-cluster dispersion (sigma) [km]", 
     main="Traceplot of sigma for the selected\ndataset analysed with poisson model")
plot(output$p_c_vec[1,],type="l",xlab='MCMC time(thinned)',ylab="Prob. of a cluster of having size 1 ( p^{(c)}_1 )", 
     main="Traceplot of p^c_1 for the selected\ndataset analysed with poisson model")
plot(output$p_c_vec[2,],type="l",,xlab='MCMC time(thinned)',ylab="Prob. of a cluster of having size 2 ( p^{(c)}_2 )", 
     main="Traceplot of p^c_2 for the selected\ndataset analysed with poisson model")
plot(output$p_c_vec[3,],type="l",xlab='MCMC time(thinned)',ylab="Prob. of a cluster of having size 3 ( p^{(c)}_3 )", 
     main="Traceplot of p^c_3 for the selected\ndataset analysed with poisson model")
# plot(output$Y_vec[1,],type="l",xlab='MCMC time(thinned)',ylab="Number of points in clusters of size 1 ( Y_1 )", 
#      main="Traceplot of Y_1 for the selected\ndataset analysed with poisson model")
# plot(output$Y_vec[2,],type="l",xlab='MCMC time(thinned)',ylab="Number of points in clusters of size 2 ( Y_2 )", 
#      main="Traceplot of Y_2 for the selected\ndataset analysed with poisson model")
# plot(output$Y_vec[3,],type="l",xlab='MCMC time(thinned)',ylab="Number of points in clusters of size 3 ( Y_3 )", 
#      main="Traceplot of Y_3 for the selected\ndataset analysed with poisson model")
# plot(output$Y_vec[4,],type="l",xlab='MCMC time(thinned)',ylab="Number of points in clusters of size 4 ( Y_4 )", 
#      main="Traceplot of Y_4 for the selected\ndataset analysed with poisson model")