# this file imports in R the dataset from the text files (from the folder whose address is
# saved as "directory_data") and stores them as a list. Each element of the
# list contains the x-y coordinates of the settlements corresponding to one placename

directory_data<-'Settlements_dataset_text_files/'

library(sp)
library(maptools)
library(rgdal)

#### DATA ACQUISITION ###############
###
# reads the txt files
names<-c('ASTON','BOLTON','BURH-STALL','BURTON','CENTRES','CHARLTON','CHESTERTON','CLAEG','DRAYTON','EATON','KINGSTON','KNIGHTON','NEWBOLD','NEWTON','NORTON','STRATTON','SUTTON','TOT','WALTON','WESTON')
imported_list<-list()
imported_list[[length(names)]]<-NA
for (i in 1:length(names)){
  imported_list[[i]]<-read.table(
    paste(directory_data,names[i],'.txt',sep=''))
}

#### DATA TRANSFORMATION ################
###
# converts the coordinates from the OS British NAtional grid system (two letters and 6 digits) to meters
# the reference system is still the one of the OS British NAtional grid system
source(paste(directory_data,'Grid_letters.R',sep=''))
points_list_C<-list()
points_list_C[[length(names)]]<-NA
for (i in 1:length(names)){
  Temp_grid_letters<-imported_list[[i]][,1]
  Temp_grid_numbers<-imported_list[[i]][,c(2,3)]
  ######## WRITE AS OSGB (numeric version of British national grid system)#########
  x_coord_large_scale<-vapply(Temp_grid_letters,function(y){
    return(Grid_letters_offset[which(y==Grid_letters_list),1])},FUN.VALUE=1)#
  y_coord_large_scale<-vapply(Temp_grid_letters,function(y){
    return(Grid_letters_offset[which(y==Grid_letters_list),2])},FUN.VALUE=1)#
  east_coord<-x_coord_large_scale+Temp_grid_numbers[,1]*100
  north_coord<-y_coord_large_scale+Temp_grid_numbers[,2]*100
  Temp_C = data.frame(x=east_coord,y=north_coord)
  coordinates(Temp_C)=~x+y
  proj4string(Temp_C)=CRS("+init=epsg:27700")#epsg:27700 is the national OSGB (British national grid system)
  points_list_C[[i]]<-Temp_C
}
num_points_vector<-NULL
for (i in 1:length(names)){
  num_points_vector[i]<-length(points_list_C[[i]])
}

#### REMOVING COUPLES #####
###
# merge together multiple records of the same settlement. Each couple (or triple) of 
# settlements with the same placename and distance smaller than tol_dist (now set to 3km)
# is replaced by a single settlement at their center. points_list_C is before merging 
# process (i.e. with couples) and points_list_NC is after (i.e. without couples)
# The couples clearly classified by historians as representing a signle settlement were
# already merged in teh text files.
tol_dist<-3000#m
remove_couples_function<-function(temp_points){
  temp_points<-t(temp_points)
  temp_dist<-as.matrix(dist(t(temp_points),diag=TRUE,upper=TRUE))
  temp_dist[upper.tri(temp_dist,diag=TRUE)]<-Inf
  temp_ind<-which(temp_dist<tol_dist,arr.ind=TRUE)
  if (length(temp_ind)==0){
    return(t(temp_points))
  }
  temp_no_couple<-temp_points[,-unique(c(temp_ind))]
  list_couples<-list(NULL)
  list_couples[[length(temp_ind[,1])+1]]<-1
  for (j in 1:length(temp_ind[,1])){
    if(any(temp_ind[j,]==temp_ind[j-1,])){
      if(any(temp_ind[j-1,]==temp_ind[j-2,])){
        list_couples[[j-2]]<-unique(c(list_couples[[j-2]],temp_ind[j,]))
      }else{
        list_couples[[j-1]]<-unique(c(list_couples[[j-1]],temp_ind[j,]))
      }
    }else{
      list_couples[[j]]<-temp_ind[j,]
    }
  }
  list_couples[[length(temp_ind[,1])+1]]<-NULL
  for (j in 1:length(temp_ind[,1])){
    if (length(list_couples[[j]])>0){
      temp_no_couple<-cbind(temp_no_couple,
                            c(mean(c(temp_points[1,list_couples[[j]]])),mean(c(temp_points[2,list_couples[[j]]]))))
      
    }
  }
  return(t(temp_no_couple))
}
points_list_NC<-list()
points_list_NC[[length(names)]]<-NA
proximity_couples<-NULL
for (i in 1:length(names)){
  points_list_NC[[i]]<-remove_couples_function(points_list_C[[i]]@coords)
  proximity_couples<-c(proximity_couples,length(points_list_C[[i]])-length(points_list_NC[[i]])/2)
}